#define MEM_PATCHSIZE 19
#define WAVE_LEN 32
#define CONVERTED 127
#define CONVERTED_ADDR 1023
#include <avr/eeprom.h>
#include "AtmPatch.h"
#include "AtmPatchBase.h"
#include "AtmOscillator.h"
#include "SsHelpers.h"
#include "Wavetable.h"

AtmPatch patches[16];
Wavetable userWaves[16];
Wavetable sineWave(32);
AtmOscillator osc(32,AtmPatch::PATCH_SIZE*16);
static const char sineData[32] = {-1,23,47,70,89,105,117,124,127,124,117,105,89,70,47,23,-1,-25,-49,-72,-91,-107,-119,-126,-128,-126,-119,-107,-91,-72,-49,-25};

void writeMemory(const void* data, void* startAddr, size_t size)
{
	eeprom_update_block(data, startAddr, size );
}

void readMemory(void* data, const void* startAddr, size_t size)
{
	eeprom_read_block(data,startAddr,size);
}

void setup()
{
	byte i,j, inp;
	unsigned int addr = 0;
	Serial.begin(9600);
	Serial.println();
	for(i=0;i<32;++i)
	{
		sineWave.setSample(i,sineData[i]);
	}
	inp = eeprom_read_byte((uint8_t*)CONVERTED_ADDR);
	if(inp==CONVERTED)
	{
		Serial.print("Already converted");
	}
	else
	{
		Serial.print("Reading");
		for(i=0;i<16;++i)
		{
			Serial.print('.');
			addr = (unsigned int)i * MEM_PATCHSIZE;
			for(j=0;j<8;++j)
			{
				inp = eeprom_read_byte((uint8_t*)addr);
				addr++;
				patches[i].setFunctionValue((j*2),inp&B00001111);
				patches[i].setFunctionValue((j*2)+1,inp>>4);
			}
			patches[i].setFunctionValue(15,i);  //force the patch num to be correct
			for(j=0;j<6;++j)
			{
				inp = eeprom_read_byte((uint8_t*)addr);
				addr++;
				patches[i].setCtrlValue(0,j,inp);
			}
			for(j=0;j<2;++j)
			{
				patches[i].setCtrlValue(1,j,patches[i].getCtrlValue(0,j));
			}
			for(j=2;j<6;++j)
			{
				inp = eeprom_read_byte((uint8_t*)addr);
				addr++;
				patches[i].setCtrlValue(1,j,inp);
			}
			inp = eeprom_read_byte((uint8_t*)addr);
			addr++;
			patches[i].setOptionValue(0,(bool)bitRead(inp,0));
			patches[i].setOptionValue(1,(bool)bitRead(inp,1));
			patches[i].setOptionValue(2,(bool)bitRead(inp,2));
			patches[i].setOptionValue(3,(bool)bitRead(inp,2));
			patches[i].setOptionValue(4,(bool)bitRead(inp,2));
			patches[i].setOptionValue(5,false);
			patches[i].setOptionValue(6,false);
			patches[i].setOptionValue(7,false);
			patches[i].setOptionValue(8,false);
			patches[i].setOptionValue(9,(bool)bitRead(inp,3));
			patches[i].setOptionValue(10,(bool)bitRead(inp,3));
			patches[i].setOptionValue(11,(bool)bitRead(inp,4));
			patches[i].setOptionValue(12,(bool)bitRead(inp,4));
			patches[i].setOptionValue(13,(bool)bitRead(inp,5));
			patches[i].setOptionValue(14,(bool)bitRead(inp,6));
			patches[i].setOptionValue(15,false);
		}
		for(i=0;i<6;++i)
		{
			Serial.print('.');
			addr = ((unsigned int)i * WAVE_LEN) + (MEM_PATCHSIZE*16);
			userWaves[i] = Wavetable(32);
			for(j=0;j<32;++j)
			{
				inp = eeprom_read_byte((uint8_t*)addr);
				addr++;
				userWaves[i].setSample(j,(char)inp);
			}
		}
		Serial.println();
		for(i=6;i<16;++i)
		{
			userWaves[i] = Wavetable(32);
			for(j=0;j<32;++j)
			{
				userWaves[i].setSample(j,sineWave.getSample(j));
			}
		}
		Serial.println("Writing");
		for(i=0;i<16;++i)
		{
			Serial.print('.');
			patches[i].writePatch(i);
		}
		for(i=0;i<16;++i)
		{
			Serial.print('.');
			for(j=0;j<32;++j)
			{
				osc.setUserWavetableSample(j,userWaves[i].getSample(j));
			}
			osc.writeUserWave(i);
		}
		eeprom_write_byte((uint8_t*)CONVERTED_ADDR,CONVERTED);  //to say that it's been done
		Serial.println();
		Serial.print("Done");
	}

}

void loop()
{

	/* add main program code here */

}
